@@ -1,4 +1,4 @@
-package Pcore::API::Whois v0.2.1;
+package Pcore::API::Whois v0.2.2;
 
 use Pcore -dist, -class;
 use Pcore::API::Whois::Server;
@@ -59,10 +59,6 @@ sub search ( $self, $domain, $cb = undef ) {
 
     state $q1 = $self->cache ? $self->dbh->query('SELECT status FROM whois WHERE domain = ? AND last_checked > ?') : undef;
 
-    state $q2 = $self->cache ? $self->dbh->query('INSERT OR IGNORE INTO whois (last_checked, status, domain) VALUES (?, ?, ?)') : undef;
-
-    state $q3 = $self->cache ? $self->dbh->query('UPDATE whois SET last_checked = ?, status = ? WHERE domain = ?') : undef;
-
     my $server_host;
 
     my @label = split /[.]/sm, P->host($domain)->canon;
@@ -134,40 +130,9 @@ sub search ( $self, $domain, $cb = undef ) {
                         $blocking_cv->($res) if $blocking_cv;
                     }
                     else {
-                        $server->request(
-                            $query,
+                        $self->_request(
+                            $server, $query,
                             sub ($res) {
-                                if ( $res->status == 200 ) {
-                                    my $recognized;
-
-                                    if ( $server->exceed_re ) {
-                                        for my $re ( $server->{exceed_re}->@* ) {
-                                            if ( $res->content->$* =~ $re ) {
-                                                $recognized = 1;
-
-                                                $res->set_status( $WHOIS_BANNED, $Pcore::API::Whois::Response::STATUS_REASON->{$WHOIS_BANNED} );
-
-                                                last;
-                                            }
-                                        }
-                                    }
-
-                                    if ( !$recognized && $server->notfound_re ) {
-                                        for my $re ( $server->{notfound_re}->@* ) {
-                                            if ( $res->content->$* =~ $re ) {
-                                                $recognized = 1;
-
-                                                $res->set_status( $WHOIS_NOT_FOUND, $Pcore::API::Whois::Response::STATUS_REASON->{$WHOIS_NOT_FOUND} );
-
-                                                last;
-                                            }
-                                        }
-                                    }
-
-                                    # cache results for successful query
-                                    $q2->do( bind => [ time, $res->status, $query ] ) || $q3->do( bind => [ time, $res->status, $query ] ) if $self->cache;
-                                }
-
                                 $cb->($res) if $cb;
 
                                 $blocking_cv->($res) if $blocking_cv;
@@ -186,6 +151,64 @@ sub search ( $self, $domain, $cb = undef ) {
     return $blocking_cv ? $blocking_cv->recv : ();
 }
 
+sub _request ( $self, $server, $query, $cb ) {
+    state $q1 = $self->cache ? $self->dbh->query('INSERT OR IGNORE INTO whois (last_checked, status, domain) VALUES (?, ?, ?)') : undef;
+
+    state $q2 = $self->cache ? $self->dbh->query('UPDATE whois SET last_checked = ?, status = ? WHERE domain = ?') : undef;
+
+    state $pool = {};
+
+    push $pool->{$query}->@*, $cb;
+
+    return if $pool->{$query}->@* > 1;
+
+    $server->request(
+        $query,
+        sub ($res) {
+            if ( $res->status == 200 ) {
+                my $recognized;
+
+                if ( $server->exceed_re ) {
+                    for my $re ( $server->{exceed_re}->@* ) {
+                        if ( $res->content->$* =~ $re ) {
+                            $recognized = 1;
+
+                            $res->set_status( $WHOIS_BANNED, $Pcore::API::Whois::Response::STATUS_REASON->{$WHOIS_BANNED} );
+
+                            last;
+                        }
+                    }
+                }
+
+                if ( !$recognized && $server->notfound_re ) {
+                    for my $re ( $server->{notfound_re}->@* ) {
+                        if ( $res->content->$* =~ $re ) {
+                            $recognized = 1;
+
+                            $res->set_status( $WHOIS_NOT_FOUND, $Pcore::API::Whois::Response::STATUS_REASON->{$WHOIS_NOT_FOUND} );
+
+                            last;
+                        }
+                    }
+                }
+
+                # cache results for successful query
+                $q1->do( bind => [ time, $res->status, $query ] ) || $q2->do( bind => [ time, $res->status, $query ] ) if $self->cache && $res->is_success;
+            }
+
+            while ( my $cb = shift $pool->{$query}->@* ) {
+                $cb->($res);
+            }
+
+            delete $pool->{$query};
+
+            return;
+        }
+    );
+
+    return;
+}
+
 sub export_supported_pub_suffixes ($self) {
     my $pub_suffix;
 
@@ -226,11 +249,9 @@ sub export_supported_pub_suffixes ($self) {
 ## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
 ## | Sev. | Lines                | Policy                                                                                                         |
 ## |======+======================+================================================================================================================|
-## |    3 | 21, 194, 211, 215    | References::ProhibitDoubleSigils - Double-sigil dereference                                                    |
+## |    3 | 21, 217, 234, 238    | References::ProhibitDoubleSigils - Double-sigil dereference                                                    |
 ## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
-## |    3 | 57                   | Subroutines::ProhibitExcessComplexity - Subroutine "search" with high complexity score (36)                    |
-## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
-## |    3 | 145, 157             | ControlStructures::ProhibitDeepNests - Code structure is deeply nested                                         |
+## |    3 | 57                   | Subroutines::ProhibitExcessComplexity - Subroutine "search" with high complexity score (24)                    |
 ## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
 ##
 ## -----SOURCE FILTER LOG END-----
